pub struct LWEParams {
pub dimension: usize,
pub log2_std_dev: i32,
}Expand description
Structure describing the security parameters for encryption with LWE ciphertexts
Attributes
dimension-the size of an LWE masklog2_std_dev-the log2 of the standard deviation used for the error normal distribution
Fields
dimension: usizelog2_std_dev: i32Implementations